@@ -84,9 +84,9 @@ class FullZeroPi(QubitBaseClass):
 
         if EC is None and ECS is None:
             raise ValueError("Argument missing: must either provide EC or ECS")
-        elif EC and ECS:
+        if EC and ECS:
             raise ValueError("Argument error: can only provide either EC or ECS")
-        elif EC:
+        if EC:
             self.EC = EC
         else:
             self.EC = 1 / (1 / ECS - 1 / self.ECJ)
@@ -120,7 +120,7 @@ class FullZeroPi(QubitBaseClass):
         )
 
     def ECS(self):
-        """ """
+        """Helper function to calculate and return `ECS`"""
         return 1 / (1 / self.EC + 1 / self.ECJ)
 
     def set_EC_via_ECS(self, ECS):
@@ -153,7 +153,8 @@ class FullZeroPi(QubitBaseClass):
         prefactor = self.omega_zeta()
         zeta_diag_hamiltonian = op.number_sparse(zeta_dim, prefactor)
 
-        hamiltonian_mat = sparse.kron(zeropi_diag_hamiltonian, sparse.identity(zeta_dim, format='dia', dtype=np.complex_))
+        hamiltonian_mat =  sparse.kron(zeropi_diag_hamiltonian,
+                                       sparse.identity(zeta_dim, format='dia', dtype=np.complex_))
         hamiltonian_mat += sparse.kron(sparse.identity(zeropi_dim, format='dia', dtype=np.complex_),
                                        zeta_diag_hamiltonian)
 
@@ -167,8 +168,7 @@ class FullZeroPi(QubitBaseClass):
 
         if return_parts:
             return [hamiltonian_mat.tocsc(), zeropi_evals, zeropi_evecs, gmat]
-        else:
-            return hamiltonian_mat.tocsc()
+        return hamiltonian_mat.tocsc()
 
     def _zeropi_operator_in_product_basis(self, zeropi_operator, zeropi_evecs=None):
         """Helper method that converts a zeropi operator into one in the product basis'
@@ -179,15 +179,15 @@ class FullZeroPi(QubitBaseClass):
         zeta_dim = self.zeta_cutoff
 
         if zeropi_evecs is None:
-            zeropi_evals, zeropi_evecs = self._zeropi.eigensys(evals_count=zeropi_dim)
+            _, zeropi_evecs = self._zeropi.eigensys(evals_count=zeropi_dim)
 
         op_eigen_basis = sparse.dia_matrix((zeropi_dim, zeropi_dim),
                                            dtype=np.complex_)  # is this guaranteed to be zero?
 
         op_zeropi = get_matrixelement_table(zeropi_operator, zeropi_evecs, real_valued=False)
-        for l1 in range(zeropi_dim):
-            for l2 in range(zeropi_dim):
-                op_eigen_basis += op_zeropi[l1, l2] * op.hubbard_sparse(l1, l2, zeropi_dim)
+        for n in range(zeropi_dim):
+            for m in range(zeropi_dim):
+                op_eigen_basis += op_zeropi[n, m] * op.hubbard_sparse(n, m, zeropi_dim)
 
         return sparse.kron(op_eigen_basis, sparse.identity(zeta_dim, format='csc', dtype=np.complex_), format='csc')
 
@@ -270,12 +270,19 @@ class FullZeroPi(QubitBaseClass):
 
         Parameters
         ----------
-        h5file: open h5py file
+        h5file_root: open h5py file
         """
         super().filewrite_params_h5(h5file_root)
         self.grid.filewrite_params_h5(h5file_root)
 
     def set_params_from_h5(self, h5file_root):
+        """Read and store parameters from open h5 file
+
+        Parameters
+        ----------
+        h5file_root: h5py.Group
+            handle to root group in open h5 file
+        """
         super().set_params_from_h5(h5file_root)
         grid_attrs = h5file_root.get('grid').attrs
         self.grid.min_val = grid_attrs['min_val']
